"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dualCmp = (chk, add) => (a, b) => {
    if (chk(a) || chk(b)) {
        return chk(a) && chk(b) && add(a, b);
    }
    return undefined;
};
const arrayEq = dualCmp(Array.isArray, (a, b) => a.length === b.length && a.every((val, i) => deepEquals(val, b[i])));
const bufferEq = typeof Buffer == "function"
    ? dualCmp(Buffer.isBuffer, (a, b) => a.equals(b))
    : undefined;
const nanEq = dualCmp(
// Do typeof check as isNaN returns true for non-number values.
(val) => typeof val == "number" && isNaN(val), () => true);
const primEq = dualCmp((val) => typeof val != "object" || val == null, (a, b) => a === b);
const propsEq = (a, b) => {
    const keysA = Object.keys(a);
    const keysB = new Set(Object.keys(b));
    return (keysA.length === keysB.size &&
        // WARNING: Object.keys([]) does not include "length" but [].hasOwnProperty("length") is true!
        // Therefore, we use set member testing instead of hasOwnProperty.
        keysA.every((k) => keysB.has(k) && deepEquals(a[k], b[k])));
};
function deepEquals(a, b) {
    // Check for NaN first as primitive checking will establish that NaN !== NaN.
    return (nanEq(a, b) ??
        primEq(a, b) ??
        arrayEq(a, b) ??
        bufferEq?.(a, b) ??
        propsEq(a, b));
}
exports.default = deepEquals;
