"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const promises_1 = require("fs/promises");
const path_1 = require("path");
const DEFAULT_IGNORED_ERRORS = ["EACCES", "ENOENT", "ENOTDIR"];
async function* recursiveReaddir(dir, filter, prefix = "", options) {
    let ents;
    try {
        ents = await promises_1.readdir(dir, { withFileTypes: true });
    }
    catch (e) {
        if ((options?.ignoredReaddirErrors ?? DEFAULT_IGNORED_ERRORS).includes(e.code)) {
            return;
        }
        throw e;
    }
    for (const ent of ents) {
        const path = path_1.join(dir, ent.name);
        if (!filter || filter.test(path)) {
            if (ent.isDirectory()) {
                yield* recursiveReaddir(path, filter, prefix + ent.name + path_1.sep, options);
            }
            else {
                yield prefix + ent.name;
            }
        }
    }
}
exports.default = recursiveReaddir;
